#! /usr/bin/env python

from setuptools import setup
from datetime import datetime
import os
import subprocess

from quixote.ptl.qx_distutils import qx_build_py

local_cfg = None
if os.path.exists('wcs/wcs_cfg.py'):
    local_cfg = file('wcs/wcs_cfg.py').read()
    os.unlink('wcs/wcs_cfg.py')

def data_tree(destdir, sourcedir):
    extensions = ['.css', '.png', '.jpeg', '.jpg', '.gif', '.xml', '.html', '.js', '.ezt', '.dat']
    r = []
    for root, dirs, files in os.walk(sourcedir):
        l = [os.path.join(root, x) for x in files if os.path.splitext(x)[1] in extensions]
        r.append( (root.replace(sourcedir, destdir, 1), l) )
        for vcs_dirname in ('CVS', '.svn', '.bzr', '.git'):
            if vcs_dirname in dirs:
                dirs.remove(vcs_dirname)
    return r

def get_version():
    from wcs import __version__ as version
    if os.path.exists('.git'):
        p = subprocess.Popen(['git','log','--oneline','HEAD~..'],
                stdout=subprocess.PIPE)
        result = p.communicate()[0]
        version += '.%s.%s' % (datetime.today().strftime('%Y%m%d'), result.split()[0])
    return version

setup(
        name = 'wcs',
        version=get_version(),
        maintainer = "Frederic Peters",
        maintainer_email = "fpeters@entrouvert.com",
        url = "http://wcs.labs.libre-entreprise.org",
        package_dir = { 'wcs': 'wcs' },
        packages = ['wcs', 'wcs.admin', 'wcs.backoffice', 'wcs.ctl',
                'wcs.ctl.Bouncers', 'wcs.forms', 'wcs.wf', 'wcs.qommon',
                'wcs.qommon.admin', 'wcs.qommon.backoffice', 'wcs.qommon.ident',
                'wcs.qommon.vendor'],
        cmdclass = {'build_py': qx_build_py},
        scripts = ['wcsctl.py'],
        data_files = data_tree('share/wcs/web/', 'data/web/') + \
                data_tree('share/wcs/themes/', 'data/themes/') + \
                data_tree('share/wcs/vendor/', 'data/vendor/') + \
                data_tree('share/wcs/qommon/', 'wcs/qommon/static/')
    )

if local_cfg:
    file('wcs/wcs_cfg.py', 'w').write(local_cfg)
