# w.c.s. - web application for online forms
# Copyright (C) 2005-2013  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from qommon.publisher import get_publisher_class
from qommon.form import *

from wcs.workflows import WorkflowStatusItem, register_item_class, \
        get_role_translation
from wcs.formdef import FormDef

from wcs.admin.fields import FieldWidget, FieldDefPage, FieldsDirectory
from wcs.formdata import get_dict_with_varnames


class WorkflowFormFieldsFormDef(FormDef):
    def __init__(self, item):
        self.name = _('Workflow Form Fields')
        self.item = item
        self.fields = []
        self.id = None

    def store(self):
        self.item.parent.parent.store()

class WorkflowFormFieldDefPage(FieldDefPage):
    section = 'workflows'
    wsf_support = False


class WorkflowFormFieldsDirectory(FieldsDirectory):
    section = 'workflows'
    support_import = False
    blacklisted_types = ['page']


class FormWorkflowStatusItem(WorkflowStatusItem):
    description = N_('Display a form')
    key = 'form'

    by = []
    formdef = None
    varname = None

    def add_parameters_widgets(self, form, parameters, prefix=''):
        if 'by' in parameters:
            form.add(WidgetList, '%sby' % prefix, title = _('To'), element_type = SingleSelectWidget,
                value = self.by,
                add_element_label = _('Add Role'),
                element_kwargs = {'render_br': False,
                                  'options': [(None, '---')] +
                                              self.get_list_of_roles(include_logged_in_users=False)})
        if 'varname' in parameters:
            form.add(StringWidget, '%svarname' % prefix, required=True,
                     title=_('Variable Name'), value=self.varname)

            form.widgets.append(HtmlWidget(htmltext('<p><a href="fields/">%s</a></p>') % _('Edit Fields')))

    def get_parameters(self):
        return ('by', 'varname')

    def q_admin_lookup(self, workflow, status, component):
        if component == 'fields':
            if not self.formdef:
                self.formdef = WorkflowFormFieldsFormDef(item=self)
            return WorkflowFormFieldsDirectory(self.formdef)
        return None

    def fill_form(self, form, formdata, user):
        self.formdef.add_fields_to_form(form)
        form.add_submit('submit', _('Submit'))

        req = get_request()
        for field in self.formdef.fields:
            if ('f%s' % field.id) in req.form:
                continue
            if not field.prefill:
                continue
            v = field.get_prefill_value()
            if v:
                form.get_widget('f%s' % field.id).set_value(v)
                req.form['f%s' % field.id] = v
            else:
                form.get_widget('f%s' % field.id).clear_error()

    def submit_form(self, form, formdata, user, evo):
        if form.get_submit() == 'submit' and not form.has_errors():
            workflow_data = {}
            for k, v in get_dict_with_varnames(
                            self.formdef.fields, self.formdef.get_data(form)).items():
                workflow_data['%s_%s' % (self.varname, k)] = v
            formdata.update_workflow_data(workflow_data)
            formdata.store()

register_item_class(FormWorkflowStatusItem)
